# Generated by Django 3.1.2 on 2020-10-19 07:18

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('units', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='customunit',
            name='key',
            field=models.CharField(blank=True, db_index=True, default=None, max_length=255, null=True),
        ),
    ]
