# Generated by Django 3.1.5 on 2021-03-18 07:13

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('rates', '0002_auto_20201013_2244'),
    ]

    operations = [
        migrations.AddIndex(
            model_name='rate',
            index=models.Index(fields=['base_currency', 'value_date'], name='rates_rate_base_cu_2508b5_idx'),
        ),
        migrations.AddIndex(
            model_name='rate',
            index=models.Index(fields=['currency', 'value_date'], name='rates_rate_currenc_4d48ff_idx'),
        ),
    ]
