{% load static %}
<!DOCTYPE html>
<html lang="en">
<head>
    <title>Geolocation</title>
    <!-- The line below is only needed for old environments like Internet Explorer and Android 4.x -->
    <style>
        .map {
            width: 100%;
            height: 400px;
        }
    </style>
    <script src="https://unpkg.com/vue"></script>
</head>
<body>
<div id="app">
    <p> [[ title ]] </p>
    <ul>
        {% for day in days %}
        <li>
            <h4> Day {{ day }}</h4>
        </li>
        {% endfor %}
    </ul>
</div>
<script>
    new Vue({
        delimiters: ['[[', ']]'],
        el: '#app',
        data: {
            title: 'Geocurrency'
        }
    })
</script>
</body>
</html>