# Generated by Django 3.1.2 on 2020-10-13 22:44

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('rates', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='rate',
            name='base_currency',
            field=models.CharField(db_index=True, default='EUR', max_length=3),
        ),
        migrations.AlterField(
            model_name='rate',
            name='currency',
            field=models.CharField(db_index=True, max_length=3),
        ),
    ]
